@@ -25,8 +25,8 @@
 struct kernel
 {
     /* Request parameters */
-    struct sockaddr_in local;
-    struct sockaddr_in remote;
+    struct sockaddr_gen local;
+    struct sockaddr_gen remote;
 
     /* Result parameters */
     avail_t av;
