@@ -90,6 +90,31 @@ program_header(FILE *fp)
 	    __DATE__, __TIME__);
 }
 
+void
+drop_root_privs(void)
+{
+    if (server_uid == NO_UID)
+    {
+	if (str2uid("nobody", &server_uid, &server_gid) < 0)
+	    server_uid = ROOT_UID;
+    }
+    
+    if (setgroups(1, &server_gid) < 0)
+	goto Error;
+    
+    if (server_gid != ROOT_GID)
+	if (setgid(server_gid) < 0)
+	    goto Error;
+    
+    if (server_uid != ROOT_UID)
+	if (setuid(server_uid) < 0)
+	    goto Error;
+
+    return;
+  Error:
+    syslog(LOG_ERR, "Error while changing user/group privileges");
+    exit(EXIT_FAILURE);
+}    
 
 int
 main(int argc, char *argv[])
@@ -119,6 +144,7 @@ main(int argc, char *argv[])
 #ifdef SIGTTOU    
     signal(SIGTTOU, SIG_IGN);
 #endif
+    signal(SIGPIPE, SIG_IGN);
     
     s_openlog(argv0, LOG_PID|LOG_ODELAY, code);
 
@@ -358,14 +384,6 @@ main(int argc, char *argv[])
 	}
     }
 
-    if (kernel_init() < 0)
-    {
-	if (debug)
-	    fprintf(stderr, "%s: failed opening kernel devices\n",
-		    argv[0]);
-	goto Exit;
-    }
-
 #ifdef HAVE_LIBDES
     if (encrypt_flag)
     {
@@ -383,18 +401,26 @@ main(int argc, char *argv[])
     }
 #endif
 
-    if (server_uid == NO_UID)
+/* Sigh - stupid Linux handles threads like... Anyway, we'll have to
+   add this kludge to work around the fact that threads in Linux can
+   have different uid's... Luckily Linux doesn't need root to get at
+   the needed information anyway. */
+#ifdef __linux__
+    drop_root_privs();
+#endif
+    
+    if (kernel_init() < 0)
     {
-	if (str2uid("nobody", &server_uid, &server_gid) < 0)
-	    server_uid = ROOT_UID;
+	if (debug)
+	    fprintf(stderr, "%s: failed opening kernel devices\n",
+		    argv[0]);
+	goto Exit;
     }
-    
-    if (server_gid != ROOT_GID)
-	setgid(server_gid);
-    
-    if (server_uid != ROOT_UID)
-	setuid(server_uid);
 
+#ifndef __linux__
+    drop_root_privs();
+#endif
+    
     timeout_init();
     request_init();
     
